@@ -3,6 +3,8 @@ package Gscan2pdf::Page;
 use 5.008005;
 use strict;
 use warnings;
+use feature 'switch';
+no if $] >= 5.018, warnings => 'experimental::smartmatch';
 use Carp;
 use Glib qw(TRUE FALSE);    # To get TRUE and FALSE
 use File::Copy;
@@ -19,12 +21,16 @@ Readonly my $CM_PER_INCH    => 2.54;
 Readonly my $MM_PER_CM      => 10;
 Readonly my $MM_PER_INCH    => $CM_PER_INCH * $MM_PER_CM;
 Readonly my $PAGE_TOLERANCE => 0.02;
+Readonly my $EMPTY_LIST     => -1;
+my $EMPTY         = q{};
+my $SPACE         = q{ };
+my $DOUBLE_QUOTES = q{"};
 
 BEGIN {
     use Exporter ();
     our ( $VERSION, @EXPORT_OK, %EXPORT_TAGS );
 
-    $VERSION = '1.3.0';
+    $VERSION = '1.3.1';
 
     use base qw(Exporter);
     %EXPORT_TAGS = ();    # eg: TAG => [ qw!name1 name2! ],
@@ -135,105 +141,137 @@ sub thaw {
 
 sub boxes {
     my ($self) = @_;
-
-    # Unfortunately, there seems to be a case (tested in t/31_ocropus_utf8.t)
-    # where decode_entities doesn't work cleanly, so encode/decode to finally
-    # get good UTF-8
-    my $hocr =
-      decode_utf8(
-        encode_utf8( HTML::Entities::decode_entities( $self->{hocr} ) ) );
-
+    my $hocr = $self->{hocr};
     if ( $hocr =~ /<body>([\s\S]*)<\/body>/xsm ) {
-        return parse_hocr($hocr);
+        my $boxes = _parse_hocr($hocr);
+        _prune_empty_branches($boxes);
+        return $boxes;
     }
-    return {
-        type => 'page',
-        bbox => [ 0, 0, $self->{w}, $self->{h} ],
-        text => $hocr
-    };
+    return [
+        {
+            type => 'page',
+            bbox => [ 0, 0, $self->{w}, $self->{h} ],
+            text => _decode_hocr($hocr)
+        }
+    ];
 }
 
-sub parse_hocr {
+# Unfortunately, there seems to be a case (tested in t/31_ocropus_utf8.t)
+# where decode_entities doesn't work cleanly, so encode/decode to finally
+# get good UTF-8
+
+sub _decode_hocr {
+    my ($hocr) = @_;
+    return decode_utf8( encode_utf8( HTML::Entities::decode_entities($hocr) ) );
+}
+
+sub _parse_hocr {
     my ($hocr) = @_;
     my $p = HTML::TokeParser->new( \$hocr );
-    my ( $lineid, $wordid ) = ( 1, 1 );
-    my ( %linedata, %worddata, $data, @boxes );
+    my ( $data, @stack, $boxes );
     while ( my $token = $p->get_token ) {
-        if ( $token->[0] eq 'S' ) {
-            my ( $tag, %attrs ) = ( $token->[1], %{ $token->[2] } );
-            if ( $tag eq 'span' and defined $attrs{class} ) {
-                if ( new_hocr_data(%attrs) ) {
+        given ( $token->[0] ) {
+            when ('S') {
+                my ( $tag, %attrs ) = ( $token->[1], %{ $token->[2] } );
 
-                    # initialize data pointer
-                    if ( $attrs{class} =~ /_word$/xsm ) {
-                        %worddata       = %linedata;
-                        $data           = \%worddata;
-                        $data->{type}   = 'word';
-                        $data->{lineid} = $linedata{id};
-                        $data->{id}     = $wordid++;
+                # new data point
+                $data = {};
+
+                if ( defined $attrs{class} and defined $attrs{title} ) {
+                    _parse_tag_data( $attrs{title}, $data );
+                    given ( $attrs{class} ) {
+                        when (/_page$/xsm) {
+                            $data->{type} = 'page';
+                            push @{$boxes}, $data;
+                        }
+                        when (/_carea$/xsm) {
+                            $data->{type} = 'column';
+                        }
+                        when (/_par$/xsm) {
+                            $data->{type} = 'para';
+                        }
+                        when (/_line$/xsm) {
+                            $data->{type} = 'line';
+                        }
+                        when (/_word$/xsm) {
+                            $data->{type} = 'word';
+                        }
+                    }
+
+                    # pick up previous pointer to add style
+                    if ( not defined $data->{type} ) {
+                        $data = $stack[-1];
+                    }
+
+                    # put information xocr_word information in parent ocr_word
+                    if (    $data->{type} eq 'word'
+                        and $stack[-1]{type} eq 'word' )
+                    {
+                        for ( keys %{$data} ) {
+                            if ( not defined $stack[-1]{$_} ) {
+                                $stack[-1]{$_} = $data->{$_};
+                            }
+                            elsif ( $_ ne 'type' ) {
+                                $logger->warn("Ignoring $_=$data->{$_}");
+                            }
+                        }
+
+                        # pick up previous pointer to add any later text
+                        $data = $stack[-1];
                     }
                     else {
-                        %linedata     = ();
-                        $data         = \%linedata;
-                        $data->{type} = 'line';
-                        $data->{id}   = $lineid++;
+                        if ( defined $attrs{id} ) {
+                            $data->{id} = $attrs{id};
+                        }
+
+                        # if we have previous data, add the new data to the
+                        # contents of the previous data point
+                        if (    defined $stack[-1]
+                            and $data != $stack[-1]
+                            and defined $data->{bbox} )
+                        {
+                            push @{ $stack[-1]{contents} }, $data;
+                        }
                     }
-                    parse_tag_data( $attrs{title}, $data );
                 }
-                elsif ( $attrs{class} eq 'ocr_cinfo' ) {
 
-                    # reset data pointer
-                    undef $data;
-                    %worddata = ();
-                    %linedata = ();
+                # pick up previous pointer
+                # so that unknown tags don't break the chain
+                else {
+                    $data = $stack[-1];
+                }
+                if ( defined $data ) {
+                    if ( $tag eq 'strong' ) { push @{ $data->{style} }, 'Bold' }
+                    if ( $tag eq 'em' ) { push @{ $data->{style} }, 'Italic' }
+                }
+
+                # put the new data point on the stack
+                push @stack, $data;
+            }
+            when ('T') {
+                if ( $token->[1] !~ /^\s*$/xsm ) {
+                    $data->{text} = _decode_hocr( $token->[1] );
+                    chomp $data->{text};
                 }
             }
-            elsif ( defined $data ) {
-                if ( $tag eq 'strong' ) { push @{ $data->{style} }, 'Bold' }
-                if ( $tag eq 'em' )     { push @{ $data->{style} }, 'Italic' }
+            when ('E') {
+
+                # up a level
+                $data = pop @stack;
             }
         }
-        elsif ( $token->[0] eq 'T' ) {
-            if ( $token->[1] !~ /^\s*$/xsm ) {
-                $data->{text} = $token->[1];
-                chomp $data->{text};
-            }
-        }
-        elsif ( $token->[0] eq 'E' ) {
 
-            # reset data pointer
-            undef $data;
-        }
-
-        if (    defined $data
-            and defined $data->{text}
-            and defined $data->{bbox} )
-        {
-            push @boxes, { %{$data} };
-        }
     }
-    return @boxes;
+    return $boxes;
 }
 
-sub new_hocr_data {
-    my %attrs = @_;
-    return (
-        (
-                 $attrs{class} eq 'ocr_line'
-              or $attrs{class} eq 'ocr_word'
-              or $attrs{class} eq 'ocrx_word'
-        )
-          and defined $attrs{title}
-    );
-}
-
-sub parse_tag_data {
+sub _parse_tag_data {
     my ( $title, $data ) = @_;
     if ( $title =~ /\bbbox\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)/xsm ) {
-        $data->{bbox} = [ $1, $2, $3, $4 ];
+        if ( $1 != $3 and $2 != $4 ) { $data->{bbox} = [ $1, $2, $3, $4 ] }
     }
     if ( $title =~ /\btextangle\s+(\d+)/xsm ) { $data->{textangle}  = $1 }
-    if ( $title =~ /\bx_wconf\s+(\d+)/xsm )   { $data->{confidence} = $1 }
+    if ( $title =~ /\bx_wconf\s+(-?\d+)/xsm ) { $data->{confidence} = $1 }
     if ( $title =~ /\bbaseline\s+(?:-?\d+(?:[.]\d+)?\s+)*(-?\d+)/xsm ) {
         $data->{baseline} = $1;
 
@@ -245,6 +283,101 @@ sub parse_tag_data {
     return;
 }
 
+sub _prune_empty_branches {
+    my ($boxes) = @_;
+    if ( defined $boxes ) {
+        my $i = 0;
+        while ( $i <= $#{$boxes} ) {
+            my $child = $boxes->[$i];
+            _prune_empty_branches( $child->{contents} );
+            if ( $#{ $child->{contents} } == $EMPTY_LIST ) {
+                delete $child->{contents};
+            }
+            if ( $#{$boxes} > $EMPTY_LIST
+                and not( defined $child->{contents} or defined $child->{text} )
+              )
+            {
+                splice @{$boxes}, $i, 1;
+            }
+            else {
+                $i++;
+            }
+        }
+    }
+    return;
+}
+
+# return hocr output as string
+
+sub string {
+    my ($self) = @_;
+    return _boxes2string( $self->boxes );
+}
+
+sub _boxes2string {
+    my ($boxes) = @_;
+    my $string = $EMPTY;
+
+    # Note y value to be able to put line breaks
+    # at appropriate positions
+    my ( $oldx, $oldy );
+    for my $box ( @{$boxes} ) {
+        if ( defined $box->{contents} ) {
+            $string .= _boxes2string( $box->{contents} );
+        }
+        if ( not defined $box->{text} ) { next }
+        my ( $x1, $y1, $x2, $y2 ) = @{ $box->{bbox} };
+        if ( defined $oldx and $x1 > $oldx ) { $string .= $SPACE }
+        if ( defined $oldy and $y1 > $oldy ) { $string .= "\n" }
+        ( $oldx, $oldy ) = ( $x1, $y1 );
+        $string .= $box->{text};
+    }
+    return $string;
+}
+
+sub djvu_text {
+    my ($self) = @_;
+    my $boxes = $self->boxes;
+    if ( defined $boxes and $#{$boxes} > $EMPTY_LIST ) {
+        my $h =
+          ( $boxes->[0]{type} eq 'page' ) ? $boxes->[0]{bbox}[-1] : $self->{h};
+        return _boxes2djvu( $boxes, 0, $h );
+    }
+    return $EMPTY;
+}
+
+sub _boxes2djvu {
+    my ( $pointer, $indent, $h ) = @_;
+    my $string = $EMPTY;
+
+    # Write the text boxes
+    for my $box ( @{$pointer} ) {
+        my ( $x1, $y1, $x2, $y2 ) = @{ $box->{bbox} };
+        if ( $indent != 0 ) { $string .= "\n" }
+        for ( 1 .. $indent ) { $string .= $SPACE }
+        $string .= sprintf "($box->{type} %d %d %d %d", $x1, $h - $y2, $x2,
+          $h - $y1;
+        if ( defined $box->{text} ) {
+            $string .= $SPACE . _escape_text( $box->{text} );
+        }
+        if ( defined $box->{contents} ) {
+            $string .= _boxes2djvu( $box->{contents}, $indent + 2, $h );
+        }
+        $string .= ')';
+    }
+    if ( $indent == 0 ) { $string .= "\n" }
+    return $string;
+}
+
+# Escape backslashes and inverted commas
+# Surround with inverted commas
+sub _escape_text {
+    my ($txt) = @_;
+    $txt =~ s/\\/\\\\/gxsm;
+    $txt =~ s/"/\\\"/gxsm;
+    return "$DOUBLE_QUOTES$txt$DOUBLE_QUOTES";
+}
+
 sub to_png {
     my ( $self, $page_sizes ) = @_;
 
